package com.iformall.service.toutiao;

import com.iformall.config.WechatOpenProperties;
import com.iformall.config.WechatRedisProperies;
import com.iformall.mapper.WxAuthorizerInfoMapper;
import com.iformall.mapper.WxComponentVerifyTicketMapper;
import com.iformall.service.toutiao.api.impl.TtOpenServiceImpl;
import com.iformall.service.wechat.FmOpenInRedisDBConfigStorage;
import com.iformall.service.wechat.handler.*;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.mp.constant.WxMpEventConstants;
import me.chanjar.weixin.open.api.impl.WxOpenMessageRouter;
import me.chanjar.weixin.open.api.impl.WxOpenServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;
import redis.clients.jedis.JedisPool;

import javax.annotation.PostConstruct;

/**
 * Stormeye WU
 */
@Service
@EnableConfigurationProperties({WechatOpenProperties.class, WechatRedisProperies.class})
public class FmTtOpenService extends TtOpenServiceImpl {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WechatOpenProperties wechatProperties;

    @Autowired
    private WechatRedisProperies redisProperies;

    @Autowired
    private WxComponentVerifyTicketMapper componentVerifyTicketMapper;

    @Autowired
    private WxAuthorizerInfoMapper authorizerInfoMapper;

    private static JedisPool pool;

    @PostConstruct
    public void init() {
        FmTtOpenInRedisDBConfigStorage configStorage = new FmTtOpenInRedisDBConfigStorage(getJedisPool(), componentVerifyTicketMapper, authorizerInfoMapper);
        configStorage.setComponentAppId(wechatProperties.getComponentAppId());
        configStorage.setComponentAppSecret(wechatProperties.getComponentSecret());
        configStorage.setComponentToken(wechatProperties.getComponentToken());
        configStorage.setComponentAesKey(wechatProperties.getComponentAesKey());
        setWxOpenConfigStorage(configStorage);
    }

    private JedisPool getJedisPool() {
        if (pool == null) {
            synchronized (FmTtOpenService.class) {
                if (pool == null) {
                    pool = new JedisPool(redisProperies, redisProperies.getHost(),
                            redisProperies.getPort(), redisProperies.getConnectionTimeout(),
                            redisProperies.getSoTimeout(), redisProperies.getPassword(),
                            redisProperies.getDatabase(), redisProperies.getClientName(),
                            redisProperies.isSsl(), redisProperies.getSslSocketFactory(),
                            redisProperies.getSslParameters(), redisProperies.getHostnameVerifier());
                }
            }
        }
        return pool;
    }
}
