package com.iformall.service.toutiao.api.impl;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.reflect.TypeToken;
import com.iformall.service.toutiao.api.TtOpenComponentService;
import com.iformall.service.toutiao.api.TtOpenMaService;
import com.iformall.service.toutiao.api.TtOpenService;
import com.iformall.service.toutiao.api.bean.TtOpenAuthorizationInfo;
import com.iformall.service.toutiao.api.bean.TtOpenGsonBuilder;
import com.iformall.service.toutiao.api.bean.TtOpenRetrieveCode;
import com.iformall.service.toutiao.api.bean.TtOpenNotifyMsg;
import com.iformall.service.toutiao.utils.MsgDecrypt;
import com.iformall.service.toutiao.utils.ServerVerification;
import me.chanjar.weixin.common.error.WxError;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.common.util.http.URIUtil;
import me.chanjar.weixin.mp.bean.result.WxMpOAuth2AccessToken;
import me.chanjar.weixin.open.api.WxOpenConfigStorage;
import me.chanjar.weixin.open.bean.*;
import me.chanjar.weixin.open.bean.result.*;
import me.chanjar.weixin.open.util.json.WxOpenGsonBuilder;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;

/**
 * @author
 */
public class TtOpenComponentServiceImpl implements TtOpenComponentService {
  private static final JsonParser JSON_PARSER = new JsonParser();
  private static final Map<String, TtOpenMaService> TT_OPEN_MA_SERVICE_MAP = new ConcurrentHashMap<>();

  protected final Logger log = LoggerFactory.getLogger(this.getClass());
  private TtOpenService ttOpenService;

  public TtOpenComponentServiceImpl(TtOpenService ttOpenService) {
    this.ttOpenService = ttOpenService;
  }

  enum RequestMethod {
	  POST,GET
  }

  @Override
  public TtOpenMaService getTtMaServiceByAppid(String appId) {
    TtOpenMaService ttOpenMaService = TT_OPEN_MA_SERVICE_MAP.get(appId);
    if (ttOpenMaService == null) {
      synchronized (TT_OPEN_MA_SERVICE_MAP) {
        ttOpenMaService = TT_OPEN_MA_SERVICE_MAP.get(appId);
        if (ttOpenMaService == null) {
          ttOpenMaService = new TtOpenMaServiceImpl(this, appId, getWxOpenConfigStorage().getWxMaConfig(appId));
          TT_OPEN_MA_SERVICE_MAP.put(appId, ttOpenMaService);
        }
      }
    }
    return ttOpenMaService;
  }

  public TtOpenService getTtOpenService() {
    return ttOpenService;
  }

  @Override
  public WxOpenConfigStorage getWxOpenConfigStorage() {
    return ttOpenService.getWxOpenConfigStorage();
  }

  @Override
  public boolean checkSignature(String timestamp, String nonce, String encrypt, String signature) {
    try {
//      log.info("-----------token="+getWxOpenConfigStorage().getComponentToken()
//              +"---------timestamp="+timestamp
//              +"---------nonce="+nonce
//              +"---------encrypt="+encrypt
//              +"---------signature="+signature
//      );
      ServerVerification verification = new ServerVerification();
      String newMsgSignature = verification.getMsgSignature(getWxOpenConfigStorage().getComponentToken(),timestamp, nonce, encrypt);
//      log.info("------------newMsgSignature="+newMsgSignature);
      return verification.verify(signature,newMsgSignature);
    } catch (Exception e) {
      this.log.error("Checking signature failed, and the reason is :" + e.getMessage());
      return false;
    }
  }

  @Override
  public TtOpenNotifyMsg decrypt(String encrypt) {
    try {
      MsgDecrypt msgDecrypt = new MsgDecrypt(getWxOpenConfigStorage().getComponentAesKey());
      String plainText = msgDecrypt.decrypt(encrypt);
      log.info("解密信息----------------------------------------"+ plainText);
      TtOpenNotifyMsg openTicket = TtOpenNotifyMsg.fromJson(plainText);
      openTicket.setComponentAppId(getWxOpenConfigStorage().getComponentAppId());
      return openTicket;
    } catch (Exception e) {
      this.log.error("Checking signature failed, and the reason is :" + e.getMessage());
      return null;
    }

  }

  @Override
  public String getComponentAccessToken(boolean forceRefresh) throws WxErrorException {
    final WxOpenConfigStorage config = this.getWxOpenConfigStorage();
    if (!config.isComponentAccessTokenExpired() && !forceRefresh) {
      return config.getComponentAccessToken();
    }
    Lock lock = config.getComponentAccessTokenLock();
    lock.lock();
    try {
      if (!config.isComponentAccessTokenExpired() && !forceRefresh) {
        return config.getComponentAccessToken();
      }
      String pream = "component_appid=" + getWxOpenConfigStorage().getComponentAppId() +
              "&component_appsecret=" + getWxOpenConfigStorage().getComponentAppSecret() +
              "&component_ticket=" + getWxOpenConfigStorage().getComponentVerifyTicket();
//      JsonObject jsonObject = new JsonObject();
//      jsonObject.addProperty("component_appid", getWxOpenConfigStorage().getComponentAppId());
//      jsonObject.addProperty("component_appsecret", getWxOpenConfigStorage().getComponentAppSecret());
//      jsonObject.addProperty("component_ticket", getWxOpenConfigStorage().getComponentVerifyTicket());

      String responseContent = this.getTtOpenService().get(API_COMPONENT_TOKEN_URL, pream);
      WxOpenComponentAccessToken componentAccessToken = WxOpenComponentAccessToken.fromJson(responseContent);
      config.updateComponentAccessToken(componentAccessToken);
      return config.getComponentAccessToken();
    } finally {
      lock.unlock();
    }
  }

  @Override
  public String post(String uri, String postData,String tokenKeyName) throws WxErrorException {
	  //"component_access_token"
    return postByCommonAccessToken(uri, postData, tokenKeyName);
  }
  
  @Override
  public String get(String uri,String tokenKeyName) throws WxErrorException {
	  //"component_access_token"
	return getByCommonAccessToken(uri, tokenKeyName);
  }
  
  private String getByCommonAccessToken(String uri, String accessTokenKey)  throws WxErrorException {
	  String componentAccessToken = getComponentAccessToken(false);
	  return excuteRequet(uri, accessTokenKey, componentAccessToken, null, true, RequestMethod.GET);
  }
  private String postByCommonAccessToken(String uri, String postData ,String accessTokenKey)  throws WxErrorException {
	  String componentAccessToken = getComponentAccessToken(false);
	  return excuteRequet(uri, accessTokenKey, componentAccessToken, postData, true, RequestMethod.POST);
  }
  
  @Override
  public String getByAppAccessToken(String appId,String uri, String accessTokenKey)  throws WxErrorException {
	  String accessToken = getAuthorizerAccessToken(appId,false);
	  return excuteRequet(uri, accessTokenKey, accessToken, null, true, RequestMethod.GET);
  }
  
  @Override
  public String postByAppAccessToken(String appId,String uri,String postData, String accessTokenKey)  throws WxErrorException {
	  String accessToken = getAuthorizerAccessToken(appId, false);
	  return excuteRequet(uri, accessTokenKey, accessToken, postData, true, RequestMethod.POST);
  }
  
  private String post(String uri, String postData, String accessTokenKey,String accessTokenValue, boolean isCommpontAccessToken) throws WxErrorException {
     return excuteRequet(uri, accessTokenKey, accessTokenValue, postData, isCommpontAccessToken, RequestMethod.POST);
  }
  
  private String get(String uri, String accessTokenKey,String accessTokenValue, boolean isCommpontAccessToken) throws WxErrorException {
	  return excuteRequet(uri, accessTokenKey, accessTokenValue, null, isCommpontAccessToken, RequestMethod.GET);
  }

  @Override
  public String getPreAuthUrl(String redirectUri) throws WxErrorException {
    return createPreAuthUrl(redirectUri);
  }

  /**
   * 创建预授权链接
   *
   * @param redirectURI
   * @return
   * @throws WxErrorException
   */
  private String createPreAuthUrl(String redirectURI) throws WxErrorException {
    String uri = API_CREATE_PREAUTHCODE_URL + "?component_appid="+getWxOpenConfigStorage().getComponentAppId();

    String responseContent = post(uri, null,"component_access_token");
    log.info(responseContent +"----------responseContent");

    JsonObject jsonObject = TtOpenGsonBuilder.create().fromJson(responseContent, JsonObject.class);

    StringBuilder preAuthUrl = new StringBuilder(String.format(COMPONENT_LOGIN_PAGE_URL,
            getWxOpenConfigStorage().getComponentAppId(),
      jsonObject.get("pre_auth_code").getAsString(),
      URIUtil.encodeURIComponent(redirectURI)));
    String preAuthUrlStr = preAuthUrl.toString();
    return preAuthUrlStr;
  }


  @Override
  public String route(final TtOpenNotifyMsg ticket) throws WxErrorException {
    if (ticket == null) {
      throw new NullPointerException("message is empty");
    }
    log.info("ticket.getMsgType()"+ticket.getMsgType());
    if (StringUtils.isNotBlank(ticket.getMsgType()) && StringUtils.equalsIgnoreCase(ticket.getMsgType(), "Ticket")) {
      getWxOpenConfigStorage().setComponentVerifyTicket(ticket.getTicket());
      return "success";
    }
    return "";
  }

  @Override
  public TtOpenAuthorizationInfo getQueryAuth(String authorizationCode) throws WxErrorException {
    String uri = API_QUERY_AUTH_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId() +
            "&authorization_code=" + authorizationCode +
            "&grant_type=app_to_tp_authorization_code";

    String responseContent = get(uri,"component_access_token");
    TtOpenAuthorizationInfo queryAuth = TtOpenGsonBuilder.create().fromJson(responseContent, TtOpenAuthorizationInfo.class);
    if (queryAuth == null) {
      return queryAuth;
    }

    if (queryAuth.getAuthorizerAccessToken() != null) {
      getWxOpenConfigStorage().updateAuthorizerAccessToken(queryAuth.getAuthorizerAppid(),
              queryAuth.getAuthorizerAccessToken(), queryAuth.getExpiresIn());
    }
    if (queryAuth.getAuthorizerRefreshToken() != null) {
      getWxOpenConfigStorage().setAuthorizerRefreshToken(queryAuth.getAuthorizerAppid(), queryAuth.getAuthorizerRefreshToken());
    }
    return queryAuth;
  }

  @Override
  public String retrieveCode(String authorizerAppid) throws WxErrorException {
    String uri = RETRIEVE_AUTHORIZER_TOKEN_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId() +
            "&authorization_appid=" + authorizerAppid;
    String responseContent = post(uri, null, "component_access_token");
    TtOpenRetrieveCode retrieveCode = TtOpenGsonBuilder.create().fromJson(responseContent, TtOpenRetrieveCode.class);
    if(retrieveCode == null){
      return null;
    }
    return retrieveCode.getAuthorizationCode();
  }

//  @Override
//  public WxOpenAuthorizerInfoResult getAuthorizerInfo(String authorizerAppid) throws WxErrorException {
//    JsonObject jsonObject = new JsonObject();
//    jsonObject.addProperty("component_appid", getWxOpenConfigStorage().getComponentAppId());
//    jsonObject.addProperty("authorizer_appid", authorizerAppid);
//    String responseContent = post(API_GET_AUTHORIZER_INFO_URL, jsonObject.toString(),"component_access_token");
//    return WxOpenGsonBuilder.create().fromJson(responseContent, WxOpenAuthorizerInfoResult.class);
//  }

  @Override
  public WxOpenAuthorizerListResult getAuthorizerList(int begin, int len) throws WxErrorException {
    begin = begin < 0 ? 0 : begin;
    len = len == 0 ? 10 : len;
    JsonObject jsonObject = new JsonObject();
    jsonObject.addProperty("component_appid", getWxOpenConfigStorage().getComponentAppId());
    jsonObject.addProperty("offset", begin);
    jsonObject.addProperty("count", len);
    String responseContent = post(API_GET_AUTHORIZER_LIST, jsonObject.toString(),"component_access_token");
    WxOpenAuthorizerListResult ret = WxOpenGsonBuilder.create().fromJson(responseContent, WxOpenAuthorizerListResult.class);
    if (ret != null && ret.getList() != null) {
      for (Map<String, String> data : ret.getList()) {
        String authorizerAppid = data.get("authorizer_appid");
        String refreshToken = data.get("refresh_token");
        if (authorizerAppid != null && refreshToken != null) {
          this.getWxOpenConfigStorage().setAuthorizerRefreshToken(authorizerAppid, refreshToken);
        }
      }
    }
    return ret;
  }

  @Override
  public WxOpenAuthorizerOptionResult getAuthorizerOption(String authorizerAppid, String optionName) throws WxErrorException {
    JsonObject jsonObject = new JsonObject();
    jsonObject.addProperty("component_appid", getWxOpenConfigStorage().getComponentAppId());
    jsonObject.addProperty("authorizer_appid", authorizerAppid);
    jsonObject.addProperty("option_name", optionName);
    String responseContent = post(API_GET_AUTHORIZER_OPTION_URL, jsonObject.toString(),"component_access_token");
    return WxOpenGsonBuilder.create().fromJson(responseContent, WxOpenAuthorizerOptionResult.class);
  }

  @Override
  public void setAuthorizerOption(String authorizerAppid, String optionName, String optionValue) throws WxErrorException {
    JsonObject jsonObject = new JsonObject();
    jsonObject.addProperty("component_appid", getWxOpenConfigStorage().getComponentAppId());
    jsonObject.addProperty("authorizer_appid", authorizerAppid);
    jsonObject.addProperty("option_name", optionName);
    jsonObject.addProperty("option_value", optionValue);
    post(API_SET_AUTHORIZER_OPTION_URL, jsonObject.toString(),"component_access_token");
  }

  @Override
  public String getAuthorizerAccessToken(String appId, boolean forceRefresh) throws WxErrorException {
    WxOpenConfigStorage config = getWxOpenConfigStorage();
    if (!config.isAuthorizerAccessTokenExpired(appId) && !forceRefresh) {
      return config.getAuthorizerAccessToken(appId);
    }
    Lock lock = config.getWxMpConfigStorage(appId).getAccessTokenLock();
    lock.lock();
    try {
      if (!config.isAuthorizerAccessTokenExpired(appId) && !forceRefresh) {
        return config.getAuthorizerAccessToken(appId);
      }

      String uri = API_AUTHORIZER_TOKEN_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId() +
              "&authorizer_refresh_token=" + getWxOpenConfigStorage().getAuthorizerRefreshToken(appId) +
              "&grant_type=app_to_tp_refresh_token";

      String responseContent = get(uri,"component_access_token");
      log.info("获取authorizerAccessToken-----------------"+responseContent);

      TtOpenAuthorizationInfo queryAuth = TtOpenGsonBuilder.create().fromJson(responseContent, TtOpenAuthorizationInfo.class);
      log.info("获取authorizerAccessToken-----------------"+TtOpenGsonBuilder.create().toJson(queryAuth));
      if (queryAuth.getAuthorizerAccessToken() != null) {
        config.updateAuthorizerAccessToken(queryAuth.getAuthorizerAppid(),
                queryAuth.getAuthorizerAccessToken(), queryAuth.getExpiresIn());
      }
      if (queryAuth.getAuthorizerRefreshToken() != null) {
        config.setAuthorizerRefreshToken(queryAuth.getAuthorizerAppid(), queryAuth.getAuthorizerRefreshToken());
      }
      return config.getAuthorizerAccessToken(appId);
    } finally {
      lock.unlock();
    }
  }

  @Override
  public WxMpOAuth2AccessToken oauth2getAccessToken(String appId, String code) throws WxErrorException {
    String url = String.format(OAUTH2_ACCESS_TOKEN_URL, appId, code, getWxOpenConfigStorage().getComponentAppId());
    String responseContent = get(url,"component_access_token");
    return WxMpOAuth2AccessToken.fromJson(responseContent);
  }

  @Override
  public WxMpOAuth2AccessToken oauth2refreshAccessToken(String appId, String refreshToken) throws WxErrorException {
    String url = String.format(OAUTH2_REFRESH_TOKEN_URL, appId, refreshToken, getWxOpenConfigStorage().getComponentAppId());
    String responseContent = get(url,"component_access_token");
    return WxMpOAuth2AccessToken.fromJson(responseContent);
  }

  @Override
  public String oauth2buildAuthorizationUrl(String appId, String redirectURI, String scope, String state) {
    return String.format(CONNECT_OAUTH2_AUTHORIZE_URL,
      appId, URIUtil.encodeURIComponent(redirectURI), scope, StringUtils.trimToEmpty(state), getWxOpenConfigStorage().getComponentAppId());
  }

  @Override
  public WxMaJscode2SessionResult miniappJscode2Session(String appId, String jsCode) throws WxErrorException {
    String url = String.format(MINIAPP_JSCODE_2_SESSION, appId, jsCode, getWxOpenConfigStorage().getComponentAppId());
    String responseContent = get(url,"component_access_token");
    return WxMaJscode2SessionResult.fromJson(responseContent);
  }

  @Override
  public List<WxOpenMaCodeTemplate> getTemplateDraftList() throws WxErrorException {
    String uri = GET_TEMPLATE_DRAFT_LIST_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId();
    String responseContent = get(uri, "component_access_token");
    JsonObject response = JSON_PARSER.parse(StringUtils.defaultString(responseContent, "{}")).getAsJsonObject();
    boolean hasDraftList = response.has("draft_list");
    if (hasDraftList) {
      return WxOpenGsonBuilder.create().fromJson(response.getAsJsonArray("draft_list"),
        new TypeToken<List<WxOpenMaCodeTemplate>>() {
        }.getType());
    } else {
      return null;
    }
  }

  @Override
  public List<WxOpenMaCodeTemplate> getTemplateList() throws WxErrorException {
    String uri = GET_TEMPLATE_LIST_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId();
    String responseContent = get(uri, "component_access_token");
    JsonObject response = JSON_PARSER.parse(StringUtils.defaultString(responseContent, "{}")).getAsJsonObject();
    boolean hasTemplateList = response.has("template_list");
    if (hasTemplateList) {
      return WxOpenGsonBuilder.create().fromJson(response.getAsJsonArray("template_list"),
        new TypeToken<List<WxOpenMaCodeTemplate>>() {
        }.getType());
    } else {
      return null;
    }
  }

  @Override
  public void addToTemplate(long draftId) throws WxErrorException {
    String uri = ADD_TO_TEMPLATE_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId();
    JsonObject param = new JsonObject();
    param.addProperty("draft_id", draftId);
    post(uri, param.toString(), "component_access_token");
  }

  @Override
  public void deleteTemplate(long templateId) throws WxErrorException {
    String uri = DELETE_TEMPLATE_URL + "?component_appid=" + getWxOpenConfigStorage().getComponentAppId();

    JsonObject param = new JsonObject();
    param.addProperty("template_id", templateId);
    post(uri, param.toString(), "component_access_token");
  }

  @Override
  public WxOpenCreateResult createOpenAccount(String appId) throws WxErrorException {
    JsonObject param = new JsonObject();
    param.addProperty("appid", appId);

    String json = postByAppAccessToken(appId, CREATE_OPEN_URL, param.toString(), "access_token");

    return WxOpenCreateResult.fromJson(json);
  }

  
  
  private String excuteRequet(String uri,String tokenKey,String tokenValue,String postData,boolean isCompontAccessToken,RequestMethod method) throws WxErrorException {
    String uriWithComponentAccessToken = uri + (uri.contains("?") ? "&" : "?") + tokenKey + "=" + tokenValue;
    log.info("uri------------"+uriWithComponentAccessToken);
    log.info("postData--------"+ postData);
	    try {
	    	if (method == RequestMethod.POST) {
	    		return getTtOpenService().post(uriWithComponentAccessToken, postData);
	    	}else {
	    		return getTtOpenService().get(uriWithComponentAccessToken, null);
	    	}
	    } catch (WxErrorException e) {
	      WxError error = e.getError();
	      /*
	       * 发生以下情况时尝试刷新access_token
	       * 40001 获取access_token时AppSecret错误，或者access_token无效
	       * 42001 access_token超时
	       * 40014 不合法的access_token，请开发者认真比对access_token的有效性（如是否过期），或查看是否正在为恰当的公众号调用接口
	       */
	      if (error.getErrorCode() == 42001 || error.getErrorCode() == 40001 || error.getErrorCode() == 40014) {
	        // 强制设置wxMpConfigStorage它的access token过期了，这样在下一次请求里就会刷新access token
	        Lock lock = this.getWxOpenConfigStorage().getComponentAccessTokenLock();
	        lock.lock();
	        try {
	          if (isCompontAccessToken && StringUtils.equals(tokenValue, this.getWxOpenConfigStorage().getComponentAccessToken())) {
	            this.getWxOpenConfigStorage().expireComponentAccessToken();
	          }
	        } catch (Exception ex) {
	        	if (isCompontAccessToken) {
	        		this.getWxOpenConfigStorage().expireComponentAccessToken();
	        	}
	        } finally {
	          lock.unlock();
	        }
	        
	        if (isCompontAccessToken && this.getWxOpenConfigStorage().autoRefreshToken()) {
	        	if (method == RequestMethod.POST) {
                 return this.post(uri, postData, tokenKey);
	        	}else {
	        		return this.get(uri, tokenKey);
	        	}
	        }
	        
	      }
	      if (error.getErrorCode() != 0) {
	        throw new WxErrorException(error, e);
	      }
	      return null;
	    }
  }
  

  @Override
  public Boolean bindOpenAccount(String appId, String openAppid) throws WxErrorException {
    JsonObject param = new JsonObject();
    param.addProperty("appid", appId);
    param.addProperty("open_appid", openAppid);

    String json = postByAppAccessToken(appId, BIND_OPEN_URL, param.toString(), "access_token");

    return WxOpenResult.fromJson(json).isSuccess();
  }


  @Override
  public Boolean unbindOpenAccount(String appId, String openAppid) throws WxErrorException {
    JsonObject param = new JsonObject();
    param.addProperty("appid", appId);
    param.addProperty("open_appid", openAppid);

    String json = postByAppAccessToken(appId, UNBIND_OPEN_URL, param.toString(), "access_token");

    return WxOpenResult.fromJson(json).isSuccess();
  }


  @Override
  public WxOpenGetResult getOpenAccount(String appId) throws WxErrorException {
    JsonObject param = new JsonObject();
    param.addProperty("appid", appId);

    String json = postByAppAccessToken(appId, GET_OPEN_URL, param.toString(), "access_token");
    return WxOpenGetResult.fromJson(json);
  }


  @Override
  public WxOpenResult fastRegisterWeapp(String name, String code, String codeType, String legalPersonaWechat, String legalPersonaName, String componentPhone) throws WxErrorException {
    JsonObject jsonObject = new JsonObject();
    jsonObject.addProperty("name", name);
    jsonObject.addProperty("code", code);
    jsonObject.addProperty("code_type", codeType);
    jsonObject.addProperty("legal_persona_wechat", legalPersonaWechat);
    jsonObject.addProperty("legal_persona_name", legalPersonaName);
    jsonObject.addProperty("component_phone", componentPhone);
    String response = post(FAST_REGISTER_WEAPP_URL, jsonObject.toString(), "component_access_token");
    return WxOpenGsonBuilder.create().fromJson(response, WxOpenResult.class);
  }

  @Override
  public WxOpenResult fastRegisterWeappSearch(String name, String legalPersonaWechat, String legalPersonaName) throws WxErrorException {
    JsonObject jsonObject = new JsonObject();
    jsonObject.addProperty("name", name);
    jsonObject.addProperty("legal_persona_wechat", legalPersonaWechat);
    jsonObject.addProperty("legal_persona_name", legalPersonaName);
    String response = post(FAST_REGISTER_WEAPP_SEARCH_URL, jsonObject.toString(), "component_access_token");
    return WxOpenGsonBuilder.create().fromJson(response, WxOpenResult.class);
  }
}
