package com.iformall.service.wechat.handler;

import com.iformall.domain.po.WxAuthorizerInfo;
import com.iformall.mapper.WxAuthorizerInfoMapper;
import com.iformall.service.WxCUserService;
import com.iformall.service.wechat.handler.AbstractHandler;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.common.session.WxSessionManager;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @author Stormeye Wu
 */
@Component
public class SubscribeHandler extends AbstractHandler {

    @Autowired
    private WxAuthorizerInfoMapper wxAuthorizerInfoMapper;

    @Autowired
    private WxCUserService userService;

    @Override
    public WxMpXmlOutMessage handle(WxMpXmlMessage wxMessage,
                                    Map<String, Object> context, WxMpService weixinService,
                                    WxSessionManager sessionManager) throws WxErrorException {
        String appId = weixinService.getWxMpConfigStorage().getAppId();

        this.logger.info("新关注用户 OPENID: " + wxMessage.getFromUser() + ", AppId: " + appId);

        WxAuthorizerInfo authQ = new WxAuthorizerInfo();
        authQ.setAuthorizerAppid(appId);
        WxAuthorizerInfo authorizerInfo = wxAuthorizerInfoMapper.findWeChatMp(authQ);

        if(authorizerInfo == null) {
            return null;
        }
        // 获取微信用户基本信息
        try {
            WxMpUser userWxInfo = weixinService.getUserService()
                    .userInfo(wxMessage.getFromUser());
            if (userWxInfo != null) {
                if(StringUtils.isNotBlank(userWxInfo.getUnionId())) {
                    this.logger.info(userWxInfo.toString());
                    userService.saveOrUpdateMpUser(userWxInfo, authorizerInfo);
                }
            }
        } catch (WxErrorException e) {
            if (e.getError().getErrorCode() == 48001) {
                this.logger.info("该公众号没有获取用户信息权限！");
            }
        }
        /*
        WxMpXmlOutMessage responseResult = null;
        try {
            responseResult = this.handleSpecial(wxMessage);
        } catch (Exception e) {
            this.logger.error(e.getMessage(), e);
        }

        if (responseResult != null) {
            return responseResult;
        }

        try {
            return new TextBuilder().build("感谢关注", wxMessage, weixinService);
        } catch (Exception e) {
            this.logger.error(e.getMessage(), e);
        }
        */

        return null;
    }

    /**
     * 处理特殊请求，比如如果是扫码进来的，可以做相应处理
     */
    private WxMpXmlOutMessage handleSpecial(WxMpXmlMessage wxMessage)
        throws Exception {
        //TODO
        return null;
    }

}
