package com.iformall.service.wechat.handler;

import com.iformall.domain.po.WxAppinfo;
import com.iformall.domain.po.WxWeappAuditStatus;
import com.iformall.enums.EnumAppType;
import com.iformall.enums.EnumWeappAuditStatus;
import com.iformall.service.MailService;
import com.iformall.service.WxAppinfoService;
import com.iformall.service.WxWeappAuditStatusService;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.session.WxSessionManager;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * @author Stormeye Wu
 */
@Component
public class WeappAuditHandler extends AbstractHandler {

    @Autowired
    private WxWeappAuditStatusService weappAuditStatusService;

    @Autowired
    private WxAppinfoService appinfoService;

    @Autowired
    private String fmAuditEmails;

    @Autowired
    private MailService mailService;

    @Override
    public WxMpXmlOutMessage handle(WxMpXmlMessage wxMessage,
                                    Map<String, Object> context, WxMpService wxMpService,
                                    WxSessionManager sessionManager) {
        String appId = wxMpService.getWxMpConfigStorage().getAppId();
        if(wxMessage.getEvent().equalsIgnoreCase(WxConsts.EventType.WEAPP_AUDIT_SUCCESS)) {
            updateAuditStatus(appId, true, new Date(wxMessage.getSuccTime()*1000), null);
        } else if(wxMessage.getEvent().equalsIgnoreCase(WxConsts.EventType.WEAPP_AUDIT_FAIL)) {
            updateAuditStatus(appId, false, new Date(wxMessage.getFailTime()*1000), wxMessage.getReason());
        }
        return null;
    }

    private void updateAuditStatus(String appId, boolean success, Date auditTime, String reason) {
        WxWeappAuditStatus auditStatus = weappAuditStatusService.getByAppId(appId);
        if (auditStatus != null) {
            if (success) {
                auditStatus.setAuditStatus(EnumWeappAuditStatus.SUCCESS.getCode());
                auditStatus.setAuditErrCode("");
            } else {
                auditStatus.setAuditStatus(EnumWeappAuditStatus.FAIL.getCode());
                auditStatus.setAuditErrCode(reason);
            }
            auditStatus.setAuditTime(auditTime);
            weappAuditStatusService.updateStatus(auditStatus);
        }

        // 发邮件
        WxAppinfo appinfo = appinfoService.getByAppId(appId);
        if (appinfo != null) {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//设置日期格式

            StringBuilder titleSB = new StringBuilder();
            if (appinfo.getType().equals(EnumAppType.C.getCode())) {
                titleSB.append("C端小程序");
            } else if (appinfo.getType().equals(EnumAppType.B.getCode())) {
                titleSB.append("B端小程序");
            } else if (appinfo.getType().equals(EnumAppType.MP_S.getCode())) {
                titleSB.append("公众号-服务号");
            } else if (appinfo.getType().equals(EnumAppType.MP_P.getCode())) {
                titleSB.append("公众号-订阅号");
            }
            titleSB.append(appinfo.getName());
            if (success) {
                titleSB.append("审核通过");
            } else {
                titleSB.append("审核未通过");
            }

            StringBuilder sb = new StringBuilder();
            sb.append(df.format(new Date())).append("\n");
            if (success) {
                sb.append("审核通过\n");
            } else {
                sb.append("审核未通过\n").append("原因").append(reason);
            }

            String[] receivers = fmAuditEmails.split(",");

            //发送邮件
            mailService.sendSimpleMail(receivers, titleSB.toString(), sb.toString());
        }
    }

}
