package com.iformall.mp.controller;

import com.iformall.common.ResultData;
import com.iformall.domain.po.WxAuthorizerInfo;
import com.iformall.mapper.WxAuthorizerInfoMapper;
import com.iformall.mp.manager.WxMpManager;
import com.iformall.service.WxCUserService;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import me.chanjar.weixin.mp.bean.result.WxMpUserList;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author Stormeye
 */
@RestController
@RequestMapping("/wx/{appId}/mem")
public class WxMemController {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private WxMpManager wxMpManager;

    @Autowired
    private WxAuthorizerInfoMapper wxAuthorizerInfoMapper;

    @Autowired
    private WxCUserService userService;

    @GetMapping("/syncAccountFansList")
    public ResultData syncAccountFansList(@PathVariable String appId) {
        final WxMpService wxService = wxMpManager.getMpService(appId);

        WxAuthorizerInfo authQ = new WxAuthorizerInfo();
        authQ.setAuthorizerAppid(appId);
        WxAuthorizerInfo authorizerInfo = wxAuthorizerInfoMapper.findWeChatMp(authQ);

        String nextOpenId = null;

        while (true) {
            WxMpUserList mpUserList;
            try {
                mpUserList = wxService.getUserService().userList(nextOpenId);
            } catch (WxErrorException e) {
                logger.error("获取粉丝openIds出错: " + e.getMessage());
                break;
            }
            logger.info(mpUserList.toString());
            getBatchUserInfos(wxService, mpUserList, authorizerInfo);
            nextOpenId = mpUserList.getNextOpenid();
            if(StringUtils.isBlank(nextOpenId)) {
                break;
            }
        }
        return new ResultData();
    }

    private void getBatchUserInfos(WxMpService wxService, WxMpUserList mpUserList, WxAuthorizerInfo authorizerInfo) {
        List<String> openIds = mpUserList.getOpenids();

        if(openIds.size() > 100) {
            int index = 0;
            int index_end = index + 100;
            while(true) {
                if(openIds.size()-1 > index_end) {
                    getBatchUserInfo(wxService, openIds.subList(index, index_end), authorizerInfo);
                } else if(openIds.size()-1 > index) {
                    getBatchUserInfo(wxService, openIds.subList(index, openIds.size()-1), authorizerInfo);
                } else {
                    break;
                }
                index += 100;
                index_end = index + 100;
            }
        } else {
            getBatchUserInfo(wxService, openIds, authorizerInfo);
        }
    }

    private void getBatchUserInfo(WxMpService wxService, List<String> openIds, WxAuthorizerInfo authorizerInfo) {
        if(openIds.size() <= 0) {
            return;
        }
        List<WxMpUser> userList = null;
        try{
            userList = wxService.getUserService().userInfoList(openIds);
        } catch (WxErrorException e) {
            logger.error("batchget userinfo list error: " + e.getMessage());
            return;
        }
        if(userList != null) {
            logger.info(userList.toString());
            for(WxMpUser mpUser: userList) {
                if(StringUtils.isNotBlank(mpUser.getUnionId())) {
                    try {
                        userService.saveOrUpdateMpUser(mpUser, authorizerInfo);
                    } catch (Exception e) {
                        logger.error("保存用户出错: " + e.getMessage());
                    }
                }
            }
        }
    }
}
