package com.iformall.mp.manager;

import com.iformall.service.wechat.FmOpenService;
import com.iformall.service.wechat.handler.*;
import me.chanjar.weixin.mp.api.WxMpMessageRouter;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.constant.WxMpEventConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import static me.chanjar.weixin.common.api.WxConsts.*;

/**
 * wechat mp configuration
 *
 * @author Stormeye
 */
@Service
public class WxMpManager {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private static Map<String, WxMpMessageRouter> myRouterMap = new ConcurrentHashMap<String, WxMpMessageRouter>();


    @Autowired
    protected FmOpenService openService;

    @Autowired
    protected LogHandler logHandler;
    @Autowired
    protected  NullHandler nullHandler;
    @Autowired
    protected  KfSessionHandler kfSessionHandler;
    @Autowired
    protected  StoreCheckNotifyHandler storeCheckNotifyHandler;
    @Autowired
    protected  LocationHandler locationHandler;
    @Autowired
    protected  MenuHandler menuHandler;
    @Autowired
    protected  MsgHandler msgHandler;
    @Autowired
    protected  UnsubscribeHandler unsubscribeHandler;
    @Autowired
    protected  SubscribeHandler subscribeHandler;

    public WxMpService getMpService(String appId) {
        return openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
    }

    public WxMpMessageRouter getRouter(String appId) {
        if (myRouterMap.containsKey(appId)) {
            logger.info("found appId : " + appId);
            return myRouterMap.get(appId);
        } else {
            WxMpService wxMpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            WxMpMessageRouter newRouter = newRouter(wxMpService);
            myRouterMap.put(appId, newRouter);
            logger.info("found new appId : " + appId);
            return newRouter;
        }
    }

    private WxMpMessageRouter newRouter(WxMpService wxMpService) {
        final WxMpMessageRouter newRouter = new WxMpMessageRouter(wxMpService);

        // 记录所有事件的日志 （异步执行）
        newRouter.rule().handler(this.logHandler).next();


        // 接收客服会话管理事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(WxMpEventConstants.CustomerService.KF_CREATE_SESSION)
                .handler(this.kfSessionHandler).end();
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(WxMpEventConstants.CustomerService.KF_CLOSE_SESSION)
                .handler(this.kfSessionHandler).end();
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(WxMpEventConstants.CustomerService.KF_SWITCH_SESSION)
                .handler(this.kfSessionHandler).end();

        // 门店审核事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(WxMpEventConstants.POI_CHECK_NOTIFY)
                .handler(this.storeCheckNotifyHandler).end();

        // 自定义菜单事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(MenuButtonType.CLICK)
                .handler(this.menuHandler).end();

        // 点击菜单连接事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(MenuButtonType.VIEW)
                .handler(this.nullHandler).end();

        // 关注事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(EventType.SUBSCRIBE)
                .handler(this.subscribeHandler).end();

        // 取消关注事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(EventType.UNSUBSCRIBE)
                .handler(this.unsubscribeHandler).end();

        // 上报地理位置事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(EventType.LOCATION)
                .handler(this.locationHandler).end();

        // 接收地理位置消息
        newRouter.rule().async(false)
                .msgType(XmlMsgType.LOCATION)
                .handler(this.locationHandler).end();

        // 扫码事件
        newRouter.rule().async(false)
                .msgType(XmlMsgType.EVENT)
                .event(EventType.SCAN)
                .handler(this.nullHandler).end();

        // 默认
        newRouter.rule().async(false).handler(this.msgHandler).end();

        return newRouter;
    }

}
